@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015
+;;  Copyright (C) 2013,2014,2015,2017
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -21,11 +21,14 @@
   #:use-module (artanis utils)
   #:use-module (artanis config)
   #:use-module (artanis server)
-  #:use-module (artanis route) 
+  #:use-module (artanis route)
+  #:use-module (artanis ssql)
   #:use-module (artanis env)
-  #:use-module (dbi dbi)
+  #:autoload (dbi dbi) (dbi-open dbi-query dbi-get_status dbi-close dbi-get_row)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 format)
   #:use-module ((rnrs) #:select (define-record-type))
+  #:use-module (srfi srfi-43)
   #:export (DB-open
             DB-close
             DB-query
@@ -34,6 +37,7 @@
             DB-get-top-row
             DB-get-n-rows
             db-conn-success?
+            get-conn-from-pool
             init-DB
             connect-db
             make-<connection>
@@ -51,14 +55,14 @@
   ;; mysql provides two modes, addr:port or socketfile
   ;; the default values:
   ;; port: 3306, addr: localhost, dbname: artanis
-  (fields dbname port addr socketfile))
+  (fields dbname addr socketfile))
 
 (define (->mysql mysql)
   (match mysql
-    (($ <mysql> ($ <db> _ username passwd) dbname port addr #f)
-     (format #f "~a:~a:~a:tcp:~a:~a" username passwd dbname addr port))
-    (($ <mysql> ($ <db> _ username passwd) dbname #f #f socketfile)
-     (format #f "~a:~a:~a:socket:~a" username passwd dbname socketfile))
+    (($ <mysql> ($ <db> _ username passwd) dbname addr #f)
+     (format #f "~a:~a::tcp:~a" username passwd addr))
+    (($ <mysql> ($ <db> _ username passwd) dbname #f socketfile)
+     (format #f "~a:~a::socket:~a" username passwd socketfile))
     (else (error 'mysql "Wrong connection config!" mysql))))
 
 (define-record-type <sqlite3>
@@ -75,13 +79,14 @@
 
 (define-record-type <postgresql>
   (parent <db>)
-  ;; postgresql need addr:port to open
-  (fields dbname port addr))
+  (fields dbname addr socketfile))
 
 (define (->postgresql postgresql)
   (match postgresql
-    (($ <postgresql> ($ <db> _ username passwd) dbname port addr)
-     (format #f "~a:~a:~a:tcp:~a:~a" username passwd dbname addr port))
+    (($ <postgresql> ($ <db> _ username passwd) dbname addr #f)
+     (format #f "~a:~a:~a:tcp:~a" username passwd dbname addr))
+    (($ <postgresql> ($ <db> _ username passwd) dbname #f socketfile)
+     (format #f "~a:~a:~a:socket:~a" username passwd dbname socketfile))
     (else (error 'postgresql "Wrong connection config!" postgresql))))
 
 ;; NOTE:
@@ -118,10 +123,12 @@
   (case-lambda* 
    ((dbd str) (%do-connect dbd str))
    ((dbd #:key (db-name "artanis") (db-username "root") (db-passwd "")
-         (proto "tcp") (host "localhost") (port 3306))
+         (addr "localhost:3306") (proto 'tcp))
     (let ((str (case dbd
-                 ((mysql) (format #f "~a:~a:~a:~a:~a:~a" db-username db-passwd db-name proto host port))
-                 ((postgresql) (format #f "~a:~a:~a:tcp:~a:~a" db-username db-passwd db-name host port))
+                 ((mysql) (format #f "~a:~a:~a:~a:~a"
+                                  db-username db-passwd db-name proto addr))
+                 ((postgresql) (format #f "~a:~a:~a:~a:~a"
+                                       db-username db-passwd db-name proto addr))
                  ((sqlite3) (format #f "~a" db-name)))))
       (%do-connect dbd str)))))
 
@@ -134,19 +141,38 @@
     ;; (get-conf 'database) should contain username passwd and connection method
     (match db
       (('mysql username passwd dbname ('socketfile socketfile))
-       (make-<mysql> username passwd dbname #f #f socketfile))
-      (('mysql username passwd dbname ('port addr port))
-       (make-<mysql> username passwd dbname port addr #f))
+       (make-<mysql> username passwd dbname #f socketfile))
+      (('mysql username passwd dbname ('tcp addr))
+       (make-<mysql> username passwd dbname addr #f))
       (('sqlite3 username passwd dbname)
        (make-<sqlite3> username passwd dbname))
-      (('postgresql username passwd dbname ('port addr port))
-       (make-<postgresql> username passwd dbname port addr))
+      (('postgresql username passwd dbname ('tcp addr))
+       (make-<postgresql> username passwd dbname addr #f))
+      (('postgresql username passwd dbname ('socketfile sock))
+       (make-<postgresql> username passwd dbname #f sock))
       (else (error new-DB "Something is wrong, invalid db!" db)))))
 
-(define (get-conn-from-pool worker)
+(define (create-new-DB-conn)
+  (let ((conn (DB-do-conn! (new-DB))))
+    (when (not (db-conn-success? conn))
+      (error init-connection-pool "Database connect failed: "
+             (db-conn-failed-reason conn)))
+    (DB-query conn (->sql use (get-conf '(db name))))
+    conn))
+
+(define (get-conn-from-pool)
   (if *conn-pool*
-      (vector-ref *conn-pool* worker)
-      (error get-conn-from-pool "Seems the *conn-pool* wasn't well initialized!" *conn-pool*)))
+      (if (queue-empty? *conn-pool*)
+          (create-new-DB-conn)
+          (queue-out! *conn-pool*))
+      (error get-conn-from-pool "Seems the *conn-pool* wasn't well initialized!"
+             *conn-pool*)))
+
+(define (recycle-DB-conn conn)
+  (if *conn-pool*
+      (queue-in! *conn-pool* conn)
+      (error recycle-DB-conn "Seems the *conn-pool* wasn't well initialized!"
+             *conn-pool*)))
 
 (define (%db-conn-stat conn ret)
   (ret (dbi-get_status (<connection>-conn conn))))
@@ -159,23 +185,20 @@
 
 (define (init-connection-pool)
   (display "connection pools are initilizing...")
-  (let ((pool-size (get-conf '(server workers)))
-        (db (new-DB)))
-    (set! *conn-pool* (make-vector pool-size))
-    (for-each
-     (lambda (i)
-       (let ((conn (DB-do-conn! db)))
-         (when (not (db-conn-success? conn))
-           (error init-connection-pool "Database connect failed: " (db-conn-failed-reason conn)))
-         (vector-set! *conn-pool* i conn)))
-     (iota pool-size))
+  (let ((wqlen (get-conf '(server wqlen))))
+    (set! *conn-pool*
+          (let ((dbconns
+                 (map
+                  (lambda (_) (create-new-DB-conn))
+                  (iota wqlen))))
+            (list->queue dbconns)))
     (display "DB pool init ok!\n")
-    (format #t "Now there's ~a pool~:p in total.~%" pool-size)))
+    (format #t "Now the size of connection pool is ~a.~%" wqlen)))
 
 ;; ---------------------conn operations-------------------------------
 ;; Actually, it's not `open', but get a conn from pool.
 (define (DB-open rc)
-  (let ((conn (get-conn-from-pool (current-worker))))
+  (let ((conn (get-conn-from-pool)))
     (<connection>-status-set! conn 'open)
     (rc-conn! rc conn)
     conn))
@@ -189,29 +212,30 @@
 (define* (DB-query conn sql #:key (check? #f))
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-query: Invalid DB connection!" conn))
+    (throw 'artanis-err 500 DB-query "Invalid DB connection!" conn))
    ((not (eq? (<connection>-status conn) 'open))
-    (throw 'artanis-err 500 "DB-query: Can't query from a closed connection!" conn))
+    (throw 'artanis-err 500 DB-query "Can't query from a closed connection!" conn))
    ((not (string? sql))
-    (throw 'artanis-err 500 "DB-query: Invalid SQL string!" sql))
+    (throw 'artanis-err 500 DB-query "Invalid SQL string!" sql))
    (else
     (db-query-debug-info sql)
     (dbi-query (<connection>-conn conn) sql)
     (when (not (db-conn-success? conn))
-      (unless check?
-        (throw 'artanis-err 500 "DB-query failed: " (db-conn-failed-reason conn))
-        (format (current-error-port) "DB-query check failed: ~a" (db-conn-failed-reason conn))))
+      (format (current-error-port) "SQL: ~a~%" sql)
+      (if check?
+          (format (current-error-port) "DB-query check failed: ~a" (db-conn-failed-reason conn))
+          (throw 'artanis-err 500 DB-query "failed reason: `~a'~%" (db-conn-failed-reason conn))))
     conn)))
 
 ;; NOTE: actually it'll never close the connection, just recycle it.
 (define (DB-close conn)
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-close: Invalid DB connection!" conn))   
+    (throw 'artanis-err 500 DB-close "Invalid DB connection!" conn))   
    ((eq? (<connection>-status conn) 'closed)
-    (throw 'artanis-err 500 "DB-close: the connection is already closed!" conn))
+    (throw 'artanis-err 500 DB-close "the connection is already closed!" conn))
    (else
-    ;; NOTE: Because of Artanis uses green-thread, all requests share the same
+    ;; NOTE: Because Artanis uses green-thread, all requests share the same
     ;;       DB connection, so it's dangerous to leave the connection to next
     ;;       request!
     ;;       We use "select NULL;" here to clear the last query, sometimes last
@@ -220,22 +244,23 @@
     ;;       be left in the <connection> object.
     ;; NOTE: "select null;" is safe and quickly to clear the last query.
     (DB-query conn "select null;")
-    (<connection>-status-set! conn 'closed))))
+    (<connection>-status-set! conn 'closed)
+    (recycle-DB-conn conn))))
 
 (define (DB-result-status conn)
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-result-status: Invalid DB connection!" conn))
+    (throw 'artanis-err 500 DB-result-status "Invalid DB connection!" conn))
    ((not (eq? (<connection>-status conn) 'open))
-    (throw 'artanis-err 500 "DB-result-status: Can't query from a closed connection!" conn))
+    (throw 'artanis-err 500 DB-result-status "Can't query from a closed connection!" conn))
    (else (dbi-get_status (<connection>-conn conn)))))
 
 (define (DB-get-all-rows conn)
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-get-all-rows: Invalid DB connection!" conn))
+    (throw 'artanis-err 500 DB-get-all-rows "Invalid DB connection!" conn))
    ((not (eq? (<connection>-status conn) 'open))
-    (throw 'artanis-err 500 "DB-get-all-rows: Can't query from a closed connection!" conn))
+    (throw 'artanis-err 500 DB-get-all-rows "Can't query from a closed connection!" conn))
    (else
     (let lp((next (dbi-get_row (<connection>-conn conn))) (result '()))
       (if next
@@ -245,17 +270,17 @@
 (define (DB-get-top-row conn)
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-get-top-row: Invalid DB connection!" conn))
+    (throw 'artanis-err 500 DB-get-top-row "Invalid DB connection!" conn))
    ((not (eq? (<connection>-status conn) 'open))
-    (throw 'artanis-err 500 "DB-get-top-row: Can't query from a closed connection!" conn))
+    (throw 'artanis-err 500 DB-get-top-row "Can't query from a closed connection!" conn))
    (else (dbi-get_row (<connection>-conn conn)))))
 
 (define (DB-get-n-rows conn n)
   (cond
    ((not (<connection>? conn))
-    (throw 'artanis-err 500 "DB-get-n-row: Invalid DB connection!" conn))
+    (throw 'artanis-err 500 DB-get-n-rows "Invalid DB connection!" conn))
    ((not (eq? (<connection>-status conn) 'open))
-    (throw 'artanis-err 500 "DB-get-n-row: Can't query from a closed connection!" conn))
+    (throw 'artanis-err 500 DB-get-n-rows "Can't query from a closed connection!" conn))
    (else
     (let lp((next (dbi-get_row (<connection>-conn conn))) (cnt 0) (result '()))
       (if (or next (< cnt n))
@@ -264,7 +289,7 @@
 ;;--------------------------------------------------------------------
 
 (define (current-connection)
-  (get-conn-from-pool (current-worker)))
+  (get-conn-from-pool))
 
 (define (init-DB)
   (init-connection-pool))
