@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015
+;;  Copyright (C) 2014,2015,2016,2017
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -46,10 +46,23 @@
             current-dbconn
             current-appname
             change-session-backend!
-            current-session-backend))
+            current-session-backend
+            protocol-add!
+            lookup-protocol
+            must-close-connection?
+            %os-kernel
+            %os-distro
+            %kernel-version
+            %system-arch
+            kernel-version>=?
+            linux-version>=?))
 
-;; WARNING: For concurrency in green-thread, all these stuffs should be immutable
-;;          IN THE RUN TIME!!!
+;; WARNING:
+;; For concurrency in green-thread, all these stuffs should be immutable
+;; when the server-core start to work!!!
+;; TODO:
+;; set a global variable when server-core started, and each accessor here
+;; should check it first.
 
 ;; table structure:
 ;; '((rule-handler-key (handler . keys)) ...)
@@ -58,11 +71,9 @@
 (define *handlers-table* (make-hash-table))
 (define *conf-hash-table* (make-hash-table))
 
-;; NOTE: pool size equals to workers (work queues)
-;; TODO: Should be pool of pool.
-;;       In principle, each worker need just one connection because of green-thread.
-;;       But async needs non-block, so we need a pool for each worker since each conn
-;;       could be scheduled when it encounters EWOULDBREAK.
+;; NOTE: The init queue size equals to (server wqlen).
+;;       If all the available DB conn were blocked, a new DB conn will be
+;;       created, and never closed but just recycled by *conn-pool*.
 (define *conn-pool* #f)
 
 (define *before-response-hook* (make-hook 2))
@@ -102,9 +113,12 @@
 (define draw:is-skip? (make-parameter #f))
 (define draw:is-quiet? (make-parameter #f))
 
+(define *null-device-output-port*
+  (open-input-file *null-device*))
+
 (define (artanis-current-output)
   (if (draw:is-quiet?)
-      (open-output-file *null-device*)
+      *null-device-output-port*
       (current-output-port)))
 
 (define *controllers-table* (make-hash-table))
@@ -116,3 +130,66 @@
 (define *session-backend* #f)
 (define (change-session-backend! x) (set! *session-backend* x))
 (define (current-session-backend) *session-backend*)
+
+(define *proto-table* (make-hash-table))
+
+(define (protocol-add! name proto)
+  (hashq-set! *proto-table* name proto))
+
+(define (lookup-protocol name)
+  (hashq-ref *proto-table* name))
+
+(define must-close-connection? (make-parameter #f))
+
+(define (%os-kernel)
+  (vector-ref (uname) 0))
+
+(define (%os-distro)
+  (vector-ref (uname) 1))
+
+(define (%kernel-version)
+  (vector-ref (uname) 2))
+
+(define (%system-arch)
+  (vector-ref (uname) 4))
+
+;; Simple version strings comparator.
+;; It is not performance-friendly, but since it is to be used once
+;;  -- not so critical.
+;; It can compare complex version strings like
+;;  linux-4.0.74-special-build-1a and linux-4.0.74-special-build-1b
+;; And, of course, can compare the following:
+;; 3.9 and 3.10
+;; or 4.5 and 4.11
+(define (version>=? s1 s2)
+  (let ((sm1 (string-match "[0-9]+" s1))
+        (sm2 (string-match "[0-9]+" s2)))
+    ;; if one of the strings doesn't contain numeric values 
+    ;;  compare as usual strings
+    (if (or (not sm1) (not sm2)) (string>=? s1 s2)
+      ;; otherwise -- compare prefixes, then numeric parts,
+      ;; then, recursively, suffixes.
+      (let ((mp1 (match:prefix sm1))
+            (mp2 (match:prefix sm2))
+            (ms1 (match:substring sm1))
+            (ms2 (match:substring sm2)))
+        (cond
+          ((string>? mp1 mp2) #t)
+          ((string<? mp1 mp2) #f)
+          ((> (string->number ms1) (string->number ms2)) #t)
+          ((< (string->number ms1) (string->number ms2)) #f)
+          (else
+            (version>=? (match:suffix sm1) (match:suffix sm2))))))))
+
+(define (kernel-version>=? vstr)
+  (version>=? (%kernel-version) vstr))
+
+(define (linux-version>=? vstr)
+  (and (string=? (%os-kernel) "Linux")
+       (kernel-version>=? vstr)))
+
+
+;; TODO: check uname here and provide features
+;; either 'epoll or 'kqueue to use in cond-expand inside ragnarok
+;;  to select proper module for event-queue implementation.
+;; 
