@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015
+;;  Copyright (C) 2013,2014,2015,2016
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -138,6 +138,9 @@
 
 (define (->postgresql-type name . args)
   (case name
+    ;; Boolean Type
+    ((boolean) (->0 name args)) ; BOOLEAN Stores TRUE or FALSE values
+
     ;; Numeric Types
     ((bigserial) (->0 name args)) ;  bigserial 8 bytes large autoincrementing integer 1 to 9223372036854775807
     ((int) (->0 name args))
@@ -213,7 +216,6 @@
     ;; CHARACTER VARYING(n) Character string. Variable length. Maximum length n
     ((char-var) (->1 "character varying" args))
     ((binary) (->1 name args)) ; BINARY(n) Binary string. Fixed-length n
-    ((boolean) (->0 name args)) ; BOOLEAN Stores TRUE or FALSE values
     ((varbinary) (->1 name args)) ; VARBINARY(n)
     ;; Binary VARYING(n) Binary string. Variable length. Maximum length n
     ((binary-var) (->1 "binary varying" args))
@@ -394,11 +396,17 @@
         (() (values kvs w))
         (else (throw 'artanis-err 500 "->kvp: invalid kargs" next)))))
   (define (->kv kvp) (srfi-1:unzip2 kvp))
+  (define (guard vl)
+    (map (lambda (s)
+           ;; TODO: drop unsafe substrings
+           s)
+         vl))
   (lambda (tname . kargs)
     (let-values (((kvp wcond) (->kvp kargs)))
       (let ((sql (if (string-null? wcond)
                      (let-values (((k v) (->kv kvp)))
-                       (->sql insert into tname k values v))
+                       (let ((sv (guard v)))
+                         (->sql insert into tname k values sv)))
                      ;; NOTE: If there's no cond string (say, `where' clauses), you MUSTN'T
                      ;;       use UPDATE because it'll effect all the records!!! In such case,
                      ;;       you should use INSERT.
@@ -412,19 +420,19 @@
 (define (make-table-getter rc/conn)
   (define (->ret ret)
     (match ret
-      ((? integer? n) (format #f "limit ~a " n))
-      ('top "limit 1 ")
+      ((? integer? n) (format #f " limit ~a " n))
+      ('top " limit 1 ")
       ('all "")
       (_ #f)))
   (define (->group-by group-by)
     (match group-by
       ((? list columns)
-       (format #f "~{~a~^,~} " columns))
+       (format #f " ~{~a~^,~} " columns))
       (_ #f)))
   (define (->order-by order-by)
     (match order-by
       ((columns ... (? (cut memq <> '(asc desc)) m))
-       (format #f "~{~a~^,~} ~a " columns m))
+       (format #f " ~{~a~^,~} ~a " columns m))
       (_ #f)))
   (define (->opts ret group-by order-by cnd foreach)
     (define-syntax-rule (-> x tox)
@@ -447,10 +455,10 @@
                   "Invalid #:foreach, should be (column (val1 val2 val3 ...))"
                   lst))))))
     (string-concatenate
-     (list (-> ret ->ret)
-           (-> group-by ->group-by)
+     (list (-> group-by ->group-by)
            (-> order-by ->order-by)
-           (cond-combine cnd foreach))))
+           (cond-combine cnd foreach)
+           (-> ret ->ret))))
   (define (->mix columns functions)
     `(,@columns ,@(map (lambda (f) (format #f "~a(~{~a~^,~})" (car f) (cdr f))) functions)))
   (lambda* (tname #:key (columns '(*)) ; get all (*) in default
