@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015
+;;  Copyright (C) 2014,2015,2016,2017
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -19,30 +19,33 @@
 
 (define-module (artanis page)
   #:use-module (artanis utils)
+  #:use-module (artanis env)
   #:use-module (artanis config)
   #:use-module (artanis cookie)
   #:use-module (artanis tpl)
   #:use-module (artanis tpl sxml)
   #:use-module (artanis db)
   #:use-module (artanis route)
-  #:use-module (artanis env)
+  #:use-module (artanis websocket)
+  #:use-module (artanis server http)
   #:use-module (srfi srfi-19)
   #:use-module (web uri)
-  #:use-module (web request)
-  #:use-module (web response)
   #:use-module (web http)
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 iconv)
+  #:use-module ((rnrs) #:select (bytevector-length bytevector?))
   #:export (params
             response-emit
             throw-auth-needed
             tpl->html
             redirect-to
-            generate-response-with-file
-            emit-response-with-file
             tpl->response
             reject-method
             response-error-emit
             server-handler
-            init-hook))
+            init-hook
+            emit-response-with-file
+            static-page-emitter))
 
 ;; the params will be searched in binding-list first, then search from qstr
 ;; TODO: qstr should be independent from rules binding.
@@ -50,32 +53,6 @@
   (or (assoc-ref (rc-bt rc) key)
       (get-from-qstr rc key)))
 
-(define (page-show file port)
-  (bv-cat (string-append (get-conf '(server syspage path)) "/" file) port))
-
-;; ENHANCE: use colored output
-(define* (log status mime req #:optional (port (current-error-port)))
-  (let* ((uri (request-uri req))
-         (path (uri-path uri))
-         (qstr (uri-query uri))
-         (method (request-method req)))
-    (format port "[Remote] ~a @ ~a~%" (remote-info req) (local-time-stamp))
-    (format port "[Request] method: ~a, path: ~a, qeury: ~a~%" method path qstr)
-    (format port "[Response] status: ~a, MIME: ~a~%~%" status mime)))
-
-(define (render-sys-page status request)
-  (define-syntax-rule (status->page s)
-    (format #f "~a.html" s))
-  (log status 'text/html request)
-  (let ((charset (get-conf '(server charset)))
-        (mtime (generate-modify-time (current-time))))
-    (values
-     (build-response #:code status
-                     #:headers `((server . ,(get-conf '(server info)))
-                                 (last-modified . ,mtime)
-                                 (content-type . (text/html (charset . ,charset)))))
-     (page-show (status->page status) #f))))
-
 (define (rc-conn-recycle rc body)
   (and=> (rc-conn rc) DB-close))
 
@@ -86,7 +63,7 @@
   (run-hook *before-response-hook* rc body))
 
 (define (init-after-request-hook)
- #t)
+  (run-after-request! detect-if-connecting-websocket))
 
 (define (init-before-response-hook)
   (run-before-response! rc-conn-recycle))
@@ -97,6 +74,11 @@
   (init-before-response-hook))
 
 (define (handler-render handler rc)
+  (define (->bytevector body)
+    (cond
+     ((bytevector? body) body)
+     ((string? body) (string->bytevector body (get-conf '(server charset))))
+     (else body))) ; just let it be checked by http-write
   (call-with-values
       (lambda ()
         (if (thunk? handler) 
@@ -105,40 +87,36 @@
     (lambda* (body #:key (pre-headers (prepare-headers '()))
                    (status 200) 
                    (mtime (generate-modify-time (current-time))))
-      (run-before-response-hooks rc body)
-      (let ((type (assoc-ref pre-headers 'content-type)))
-        (and type (log status (car type) (rc-req rc))))
-      (values
-       (build-response #:code status
-                       #:headers `((server . ,(get-conf '(server info)))
-                                   (last-modified . ,mtime)
-                                   ,@pre-headers 
-                                   ,@(generate-cookies (rc-set-cookie rc))))
-       ;; NOTE: For inner-server, sanitize-response will handle 'HEAD method
-       ;;       though rc-method is 'GET when request-method is 'HEAD,
-       ;;       sanitize-response only checks method from request
-       body))))
-
-(define (format-status-page status request)
-  (format (current-error-port) "[EXCEPTION] ~a is abnormal request, status: ~a, "
-          (uri-path (request-uri request)) status)
-  (display "rendering a sys page for it...\n" (current-error-port)) 
-  (render-sys-page status request))
+      (let* ((reformed-body (->bytevector body))
+             (response
+              (build-response #:code status
+                              #:headers `((server . ,(get-conf '(server info)))
+                                          (last-modified . ,mtime)
+                                          ,(gen-content-length reformed-body)
+                                          ,@pre-headers 
+                                          ,@(generate-cookies (rc-set-cookie rc))))))
+        (run-before-response-hooks rc body)
+        (let ((type (assq-ref pre-headers 'content-type)))
+          (and type (artanis-log 'client status (car type) #:request (rc-req rc))))
+        ;; NOTE: For inner-server, sanitize-response will handle 'HEAD method
+        ;;       though rc-method is 'GET when request-method is 'HEAD,
+        ;;       sanitize-response only checks method from request.
+        (if (eq? (get-conf '(server engine)) 'guile)
+            (values response reformed-body)
+            (values response reformed-body
+                    ;; NOTE: return the status while handling the request.
+                    'ok))))))
 
 (define (work-with-request request body)
+  ;;(DEBUG "work with request~%")
   (catch 'artanis-err
     (lambda ()
       (let* ((rc (new-route-context request body))
              (handler (rc-handler rc)))
-        (if handler 
+        (if handler
             (handler-render handler rc)
-            (render-sys-page 404 rc))))
-    (lambda (k . e)
-      (let ((status (car e))
-            (reason (cadr e))
-            (info (caddr e)))
-        (format (current-error-port) "[ERR Reason]: ~a ~a~%" reason info)
-        (format-status-page status request)))))
+            (render-sys-page 'client 404 rc))))
+    (make-unstop-exception-handler (exception-from-client request))))
 
 (define (response-emit-error status)
   (response-emit "" #:status status))
@@ -147,46 +125,21 @@
 (define* (response-emit body #:key (status 200) 
                         (headers '())
                         (mtime (current-time)))
-  ;;(format #t "headers: ~a~%" headers)
+  (DEBUG "Response emit headers: ~a~%" headers)
   (values body #:pre-headers (prepare-headers headers) #:status status
           #:mtime (generate-modify-time mtime)))
 
 (define (throw-auth-needed)
-  (response-emit "" #:status 401 #:headers '((WWW-Authenticate . "Basic realm=\"Secure Area\""))))
+  (response-emit
+   ""
+   #:status 401
+   #:headers '((WWW-Authenticate . "Basic realm=\"Secure Area\""))))
 
 (define (server-handler request request-body)
   ;; ENHANCE: could put some stat hook here
   (run-after-request-hooks request request-body)
   (work-with-request request request-body))
 
-;; proc must return the content-in-bytevector
-(define (generate-response-with-file filename proc)
-  (if (file-exists? filename)
-      (let* ((st (stat filename))
-             ;; NOTE: we use ctime for last-modified time
-             (mtime (make-time time-utc (stat:ctime st) (stat:ctimensec st)))
-             (port (open-input-file filename))
-             (mime (guess-mime filename)))
-        (values mtime 200 (proc port) mime))
-      (throw 'artanis-err 404 "Static file doesn't exist:" filename)))
-
-;; emit static file with no cache(ETag)
-(define* (emit-response-with-file filename #:optional (headers '()))
-  (define (gen-sfile-cache)
-    (list 'public (cons 'max-age (get-conf '(cache maxage)))))
-  (call-with-values
-      (lambda ()
-        (generate-response-with-file filename (lambda (p) (bv-cat p #f))))
-    (lambda (mtime status bv mime)
-      (cond
-       ((= status 200) 
-        (response-emit bv #:status status 
-                       #:headers `((content-type . ,(list mime))
-                                   (cache-control . ,(gen-sfile-cache))
-                                   ,@headers)
-                       #:mtime mtime))
-       (else (response-emit bv #:status status))))))
-
 (define-syntax-rule (tpl->response sxml/file ...)
   (let ((html (tpl->html sxml/file ...)))
     (if html
@@ -213,3 +166,43 @@
 
 (define (reject-method method)
   (throw 'artanis-err 405 "Method is not allowed" method))
+
+;; proc must return the content-in-bytevector
+(define (generate-response-with-file filename file-sender)
+  (let* ((st (stat filename))
+         ;; NOTE: we use ctime for last-modified time
+         (mtime (make-time time-utc (stat:ctime st) (stat:ctimensec st)))
+         (mime (guess-mime filename)))
+    (values mtime 200 file-sender mime)))
+
+;; emit static file with no cache(ETag)
+(define* (emit-response-with-file filename out #:optional (headers '()))
+  (when (not (file-exists? filename))
+    (throw 'artanis-err 404 emit-response-with-file
+           "Static file `~a' doesn't exist!" filename))
+  (call-with-values
+      (lambda ()
+        (let* ((in (open-input-file filename))
+               (size (stat:size (stat filename))))
+         (generate-response-with-file
+          filename
+          (make-file-sender
+           size
+           (lambda ()
+             ;; TODO: support trunked length requesting for continously downloading
+             (sendfile out in size)
+             (force-output out)
+             (close in))))))
+    (lambda (mtime status body mime)
+      (cond
+       ((= status 200)
+        (response-emit body #:status status
+                       #:headers `((content-type . ,(list mime))
+                                   ,@headers)
+                       #:mtime mtime))
+       (else (response-emit body #:status status))))))
+
+;; When you don't want to use cache, use static-page-emitter.
+(define (static-page-emitter rc)
+  (emit-response-with-file (static-filename (rc-path rc))
+                           (request-port (rc-req rc))))
