@@ -40,7 +40,8 @@
 ;; 4. DSL should handle key-value or list for users easily.
 ;; 5. DSL should handle Stored-Procedure for specific DB implementation (as possible).
 ;; 6. There's no ORM, but FPRM. The only difference is that you never see
-;;    Classes but Closures. Hmm...what's the essential difference? Stateless, of course!
+;;    Classes but Closures. Hmm...what's the essential difference?
+;;    We should try to be stateless, it's still experimental.
 
 ;; NOTE: sql-mapping-maker returns conn object, users have to get certain rows as will.
 (define (sql-mapping-maker mode rule keys)
@@ -70,7 +71,8 @@
          (set! post-data (generate-kv-from-post-qstr (rc-body rc)))))
   (define (post-ref key) (and=> (assoc-ref post-data key) car))
   (define (table-checker rc sql)
-    (string=? (crypto (post-ref passwd)) (->passwd rc sql)))
+    (let ((pw (->passwd rc sql)))
+      (and pw (string=? (crypto (post-ref passwd)) pw))))
   (define customed-basic-checker #f)
   (define (basic-checker rc p sql)
     (format #t "~a, ~a~%" p sql)
@@ -82,24 +84,25 @@
        (set! passwd passwd-field)
        (set! username username-field)
        (lambda (u)
-         (->sql select (passwd-field) from table (where (string->keyword username-field) u))))
+         (->sql select (list passwd) from table (where (string->keyword username) u))))
       (`(table ,table ,username-field ,passwd-field ,crypto-proc)
        (set! mode 'table-specified-fields)
        (set! crypto crypto-proc)
        (set! passwd passwd-field)
        (set! username username-field)
        (lambda (u)
-         (->sql select (passwd-field) from table (where (string->keyword username-field) u))))
+         (->sql select (list passwd) from table (where (string->keyword username) u))))
       (`(table ,table ,crypto-proc)
        (set! mode 'table)
        (set! crypto crypto-proc)
        (lambda (u)
-         (->sql select (passwd) from table (where (string->keyword username) u))))
+         (->sql select (list passwd) from table (where (string->keyword username) u))))
       (`(basic ,table ,username-field ,passwd-field)
-       (set! passwd-field passwd)
+       (set! username username-field)
+       (set! passwd passwd-field)
        (set! mode 'basic)
        (lambda (u)
-         (->sql select (passwd) from table (where (string->keyword username-field) u))))
+         (->sql select (list passwd) from table (where (string->keyword username) u))))
       (`(basic ,checker)
        (set! mode 'basic)
        (set! customed-basic-checker checker)
