@@ -38,7 +38,7 @@
                json-parser-port
                json))
 
-(define *word-re* (string->sre "^[a-zA-Z0-9_]+$"))
+(define *word-re* (string->irregex "^[a-zA-Z0-9_]+$"))
 
 (define (make-json-checker fix re err)
   (lambda (jstr)
@@ -48,7 +48,7 @@
 
 ;; check whether JSON object is an array, it is considered to be harmful to
 ;; return an array directly.
-(define *array-re* (string->sre "^\\[[^\\]]*\\]$"))
+(define *array-re* (string->irregex "^\\[[^\\]]*\\]$"))
 (define not-an-array
   (make-json-checker not *array-re* "[ERROR] return JSON as an array is dangerous!~%"))
 
@@ -58,7 +58,7 @@
 (define (validate-json jstr)
   (if (every (lambda (checker) (checker jstr)) *checker-list*)
       jstr
-      (throw 'artanis-err 500 "JSON safe check didn't pass!" jstr)))
+      (throw 'artanis-err 500 validate-json "JSON safe check didn't pass!" jstr)))
 
 ;; NOTE: I'll let you specify the regexp for validating your own JSONP in case
 ;;       the default regexp is not enough for you, but AT YOUR OWN RISK!!!
@@ -70,5 +70,6 @@
     (if (irregex-match jsonp-regexp jsonp)
         (format #f "~a(~a)" jsonp (validate-json (scm->json-string sxml)))
         ;; if jsonp contains a non-word character, this could be an XSS attack.
-        (throw 'artanis-err 400 "Invalid JSONP, possibly be an XSS attack!" jsonp)))
+        (throw 'artanis-err 400 ->json-string
+               "Invalid JSONP, possibly be an XSS attack!" jsonp)))
    (else ((if securty-check? validate-json identity) (scm->json-string sxml)))))
